@@ -681,7 +681,7 @@ xpmParseColors (XpmData * data, unsigned int ncolors, unsigned int cpp,
 	  curkey = 0;
 	  lastwaskey = 0;
 	  *curbuf = '\0';	/* init curbuf */
-	  while (l = xpmNextWord (data, buf, BUFSIZ))
+	  while ((l = xpmNextWord (data, buf, BUFSIZ)) != NULL)
 	    {
 	      if (!lastwaskey)
 		{
@@ -763,7 +763,7 @@ xpmParseColors (XpmData * data, unsigned int ncolors, unsigned int cpp,
 	   */
 	  xpmNextString (data);	/* get to the next string */
 	  *curbuf = '\0';	/* init curbuf */
-	  while (l = xpmNextWord (data, buf, BUFSIZ))
+	  while ((l = xpmNextWord (data, buf, BUFSIZ)) != NULL)
 	    {
 	      if (*curbuf != '\0')
 		strcat (curbuf, " ");	/* append space */
