@@ -12,7 +12,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -60,7 +60,7 @@ oop_array_registry;
 
 /* The registry of OOPs which have been passed to C code.  Implemented
    as a red-black tree.  The registry is examined at GC time to ensure
-   that OOPs that C code knows about don't go away. */
+   that OOPs that C code knows about don't go away.  */
 static oop_registry *oop_registry_root;
 static oop_array_registry *oop_array_registry_root;
 
@@ -91,7 +91,16 @@ VMProxy gst_interpreter_proxy = {
 /* Smalltalk process support */
   _gst_async_signal, _gst_sync_wait, _gst_async_signal_and_unregister,
 
-  _gst_register_oop_array, _gst_unregister_oop_array
+  _gst_register_oop_array, _gst_unregister_oop_array,
+
+/* Convert Smalltalk datatypes to C data types (2) */
+  _gst_oop_to_long_double, _gst_long_double_to_oop,
+
+  _gst_get_object_class, _gst_get_superclass,
+  _gst_class_is_kind_of, _gst_object_is_kind_of,
+  _gst_perform, _gst_perform_with, _gst_class_implements_selector,
+  _gst_class_can_understand, _gst_responds_to,
+  _gst_oop_size, _gst_oop_at, _gst_oop_at_put
 };
 
 OOP
@@ -342,6 +351,11 @@ _gst_msg_sendf (PTR resultPtr,
 	    IS_NIL (result) ? 0.0 : _gst_oop_to_float (result);
 	  break;
 
+	case 'F':
+	  *(long double *) resultPtr =
+	    IS_NIL (result) ? 0.0 : _gst_oop_to_long_double (result);
+	  break;
+
 	case 'v':		/* don't care about the result */
 	  break;		/* "v" for "void" */
 
@@ -378,9 +392,8 @@ _gst_eval_code (const char *str)
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
 
-  _gst_compile_code = true;
   _gst_push_cstring (str);
-  _gst_parse_stream ();
+  _gst_parse_stream (false);
   _gst_pop_stream (true);
 }
 
@@ -393,9 +406,8 @@ _gst_eval_expr (const char *str)
   if (!_gst_smalltalk_initialized)
     gst_init_smalltalk ();
 
-  _gst_compile_code = false;
   _gst_push_cstring (str);
-  _gst_parse_stream ();
+  _gst_parse_stream (false);
   _gst_pop_stream (true);
   result = _gst_last_returned_value;
 
@@ -409,7 +421,7 @@ _gst_object_alloc (OOP class_oop,
   OOP oop;
 
   if (CLASS_IS_INDEXABLE (class_oop))
-    instantiate_with (class_oop, (unsigned long) size, &oop);
+    instantiate_with (class_oop, size, &oop);
   else
     instantiate (class_oop, &oop);
 
@@ -459,6 +471,12 @@ _gst_id_to_oop (long int i)
   return (OOP_AT (i));
 }
 
+OOP
+_gst_long_double_to_oop (long double f)
+{
+  return (INC_ADD_OOP (floatq_new (f)));
+}
+
 OOP
 _gst_float_to_oop (double f)
 {
@@ -549,19 +567,6 @@ _gst_set_c_object (OOP oop, PTR co)
 
   SET_COBJECT_VALUE(oop, co);
 }
-
-OOP
-_gst_c_object_to_typed_oop (PTR co,
-			    OOP typeOOP)
-{
-  if (!_gst_smalltalk_initialized)
-    gst_init_smalltalk ();
-
-  if (co == NULL)
-    return (_gst_nil_oop);
-  else
-    return (INC_ADD_OOP (_gst_c_object_new_typed (co, typeOOP)));
-}
 
 
 /***********************************************************************
@@ -632,6 +637,22 @@ _gst_oop_to_float (OOP oop)
     return 0.0 / 0.0;
 }
 
+long double
+_gst_oop_to_long_double (OOP oop)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  if (IS_CLASS (oop, _gst_floatd_class))
+    return (FLOATD_OOP_VALUE (oop));
+  else if (IS_CLASS (oop, _gst_floate_class))
+    return (FLOATE_OOP_VALUE (oop));
+  else if (IS_CLASS (oop, _gst_floatq_class))
+    return (FLOATQ_OOP_VALUE (oop));
+  else
+    return 0.0 / 0.0;
+}
+
 int
 _gst_oop_to_bool (OOP oop)
 {
@@ -685,6 +706,168 @@ _gst_oop_to_cobject (OOP oop)
   else
     return (COBJECT_VALUE (oop));
 }
+
+OOP
+_gst_get_object_class (OOP oop)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return OOP_INT_CLASS (oop);
+}
+
+OOP
+_gst_get_superclass (OOP oop)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  /* Quick tests for "class-ness".  */
+  assert (IS_OOP (oop));
+  assert (OOP_CLASS (oop) == _gst_behavior_class
+	  || OOP_CLASS (OOP_CLASS (oop)) == _gst_metaclass_class);
+
+  return SUPERCLASS (oop);
+}
+
+mst_Boolean
+_gst_class_is_kind_of (OOP candidate, OOP superclass)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  /* Quick tests for "class-ness".  */
+  assert (IS_OOP (candidate) && IS_OOP (superclass));
+  assert (OOP_CLASS (candidate) == _gst_behavior_class
+	  || OOP_CLASS (OOP_CLASS (candidate)) == _gst_metaclass_class);
+
+  if (superclass == _gst_nil_oop || candidate == superclass)
+    return true;
+
+  assert (OOP_CLASS (superclass) == _gst_behavior_class
+	  || OOP_CLASS (OOP_CLASS (superclass)) == _gst_metaclass_class);
+
+  return is_a_kind_of (candidate, superclass);
+}
+
+
+mst_Boolean
+_gst_object_is_kind_of (OOP candidate, OOP superclass)
+{
+  OOP its_class;
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  if (IS_INT (candidate))
+    {
+      its_class = _gst_small_integer_class;
+      if (superclass == _gst_small_integer_class
+	  || superclass == _gst_object_class)
+	return true;
+    }
+  else
+    its_class = OOP_CLASS (candidate);
+
+  if (superclass == _gst_nil_oop || its_class == superclass)
+    return true;
+
+  /* Quick tests for "class-ness".  */
+  assert (IS_OOP (superclass));
+  assert (OOP_CLASS (superclass) == _gst_behavior_class
+	  || OOP_CLASS (OOP_CLASS (superclass)) == _gst_metaclass_class);
+
+  return is_a_kind_of (its_class, superclass);
+}
+
+OOP
+_gst_perform (OOP receiver, OOP selector)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return _gst_nvmsg_send (receiver, selector, NULL, 0);
+}
+
+OOP
+_gst_perform_with (OOP receiver, OOP selector, OOP arg)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return _gst_nvmsg_send (receiver, selector, &arg, 1);
+}
+
+mst_Boolean
+_gst_class_implements_selector (OOP classOOP, OOP selector)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  assert (IS_OOP (classOOP));
+  assert (OOP_CLASS (classOOP) == _gst_behavior_class
+          || OOP_CLASS (OOP_CLASS (classOOP)) == _gst_metaclass_class);
+
+  return _gst_find_class_method (classOOP, selector) != _gst_nil_oop;
+}
+
+mst_Boolean
+_gst_class_can_understand (OOP classOOP, OOP selector)
+{
+  method_cache_entry dummy;
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  /* Quick test for "class-ness".  */
+  assert (IS_OOP (classOOP));
+  assert (OOP_CLASS (classOOP) == _gst_behavior_class
+          || OOP_CLASS (OOP_CLASS (classOOP)) == _gst_metaclass_class);
+
+  return _gst_find_method (classOOP, selector, &dummy);
+}
+
+mst_Boolean
+_gst_responds_to (OOP oop, OOP selector)
+{
+  method_cache_entry dummy;
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return _gst_find_method (OOP_INT_CLASS (oop), selector, &dummy);
+}
+
+size_t
+_gst_oop_size (OOP oop)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return NUM_INDEXABLE_FIELDS (oop);
+}
+
+OOP
+_gst_oop_at (OOP oop, size_t index)
+{
+  OOP result;
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  result = index_oop (oop, index + 1);
+  assert (result);
+  return result;
+}
+
+OOP
+_gst_oop_at_put (OOP oop, size_t index, OOP new)
+{
+  OOP old;
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  old = index_oop (oop, index + 1);
+  assert (old);
+  index_oop_put (oop, index, new);
+  return old;
+}
 
 
 
@@ -736,7 +919,7 @@ _gst_unregister_oop (OOP oop)
   oop_registry *entry = oop_registry_root;
 
   /* Speed things up, this will never be in the registry (but we allow
-     it to simplify client code). */
+     it to simplify client code).  */
   if (!oop)
     return;
 
@@ -817,7 +1000,7 @@ _gst_copy_registered_oops (void)
   rb_node_t *node;
   rb_traverse_t t;
 
-  /* Walk the OOP registry... */
+  /* Walk the OOP registry...  */
   for (node = rb_first(&(oop_registry_root->rb), &t); 
        node; node = rb_next(&t))
     {
@@ -825,14 +1008,14 @@ _gst_copy_registered_oops (void)
       MAYBE_COPY_OOP (k->oop);
     }
 
-  /* ...and then the OOP-array registry. */
+  /* ...and then the OOP-array registry.  */
   for (node = rb_first(&(oop_array_registry_root->rb), &t); 
        node; node = rb_next(&t))
     {
       oop_array_registry *k = (oop_array_registry *) node;
 
       /* Dereference the pointers in the tree to obtain where the array
-	 lies. */
+	 lies.  */
       OOP *first = *(k->first);
       OOP *last = *(k->last);
       COPY_OOP_RANGE (first, last);
@@ -845,7 +1028,7 @@ _gst_mark_registered_oops (void)
   rb_node_t *node;
   rb_traverse_t t;
 
-  /* Walk the OOP registry... */
+  /* Walk the OOP registry...  */
   for (node = rb_first(&(oop_registry_root->rb), &t); 
        node; node = rb_next(&t))
     {
@@ -853,14 +1036,14 @@ _gst_mark_registered_oops (void)
       MAYBE_MARK_OOP (k->oop);
     }
 
-  /* ...and then the OOP-array registry. */
+  /* ...and then the OOP-array registry.  */
   for (node = rb_first(&(oop_array_registry_root->rb), &t); 
        node; node = rb_next(&t))
     {
       oop_array_registry *k = (oop_array_registry *) node;
 
       /* Dereference the pointers in the tree to obtain where the array
-	 lies. */
+	 lies.  */
       OOP *first = *(k->first);
       OOP *last = *(k->last);
       MARK_OOP_RANGE (first, last);
@@ -873,4 +1056,31 @@ _gst_init_vmproxy (void)
   gst_interpreter_proxy.nilOOP = _gst_nil_oop;
   gst_interpreter_proxy.trueOOP = _gst_true_oop;
   gst_interpreter_proxy.falseOOP = _gst_false_oop;
+
+  gst_interpreter_proxy.objectClass = _gst_object_class;
+  gst_interpreter_proxy.arrayClass = _gst_array_class;
+  gst_interpreter_proxy.stringClass = _gst_string_class;
+  gst_interpreter_proxy.characterClass = _gst_char_class;
+  gst_interpreter_proxy.smallIntegerClass = _gst_small_integer_class;
+  gst_interpreter_proxy.floatDClass = _gst_floatd_class;
+  gst_interpreter_proxy.floatEClass = _gst_floate_class;
+  gst_interpreter_proxy.byteArrayClass = _gst_byte_array_class;
+  gst_interpreter_proxy.objectMemoryClass = _gst_object_memory_class;
+  gst_interpreter_proxy.classClass = _gst_class_class;
+  gst_interpreter_proxy.behaviorClass = _gst_behavior_class;
+  gst_interpreter_proxy.blockClosureClass = _gst_block_closure_class;
+  gst_interpreter_proxy.contextPartClass = _gst_context_part_class;
+  gst_interpreter_proxy.blockContextClass = _gst_block_context_class;
+  gst_interpreter_proxy.methodContextClass = _gst_method_context_class;
+  gst_interpreter_proxy.compiledMethodClass = _gst_compiled_method_class;
+  gst_interpreter_proxy.compiledBlockClass = _gst_compiled_block_class;
+  gst_interpreter_proxy.fileDescriptorClass = _gst_file_descriptor_class;
+  gst_interpreter_proxy.fileStreamClass = _gst_file_stream_class;
+  gst_interpreter_proxy.processClass = _gst_process_class;
+  gst_interpreter_proxy.semaphoreClass = _gst_semaphore_class;
+  gst_interpreter_proxy.cObjectClass = _gst_c_object_class;
+
+  /* And system objects.  */
+  gst_interpreter_proxy.processorOOP = _gst_processor_oop;
+
 }
