@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2001, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright 2001, 2002, 2003, 2005, 2006 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -24,7 +24,7 @@
  *
  * You should have received a copy of the GNU General Public License along with
  * GNU Smalltalk; see the file COPYING.	 If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  ***********************************************************************/
 
@@ -51,12 +51,12 @@ static polling_queue *head, **p_tail_next;
 
 /* This variable holds a variable-sized array of pollfd structures.
    NUM_USED_POLLFDS of the total NUM_TOTAL_POLLFDS items available are
-   being used. */
+   being used.  */
 static struct pollfd *pollfds;
 static int num_used_pollfds, num_total_pollfds;
 
 /* Holds the semaphores to be signaled when the operating system sends
-   us a C-style signal. */
+   us a C-style signal.  */
 static volatile OOP sem_int_vec[NSIG];
 
 /* These are the signal handlers that we install to process
@@ -155,6 +155,7 @@ _gst_sync_file_polling (int fd,
   do
     {
       errno = 0;
+      pfd.revents = 0;
       result = poll (&pfd, 1, 0);
     }
   while ((result == -1) && (errno == EINTR));
@@ -193,7 +194,7 @@ file_polling_handler (int sig)
 	return;
 
       num_used_pollfds = 0;
-      for (node = head, pprev = &head; node && n; node = *pprev)
+      for (node = head, pprev = &head; node; node = *pprev)
 	{
 	  struct pollfd *poll = &pollfds[node->poll];
 
@@ -209,7 +210,6 @@ file_polling_handler (int sig)
 		p_tail_next = pprev;
 
 	      xfree (node);
-	      n--;
 	    }
 	  else
 	    {
@@ -272,7 +272,7 @@ _gst_async_file_polling (int fd,
 
   /* Now check if I/O was made possible while setting up our machinery...
      If so, exit; otherwise, wait on the semaphore and the SIGIO
-     will wake us up. */
+     will wake us up.  */
 
   result = _gst_sync_file_polling (fd, cond);
   if (result == 0)
