@@ -35,14 +35,111 @@
 #include "config.h"
 #include "gst.h"
 
-#if SIZEOF_CHAR_P != SIZEOF_LONG
-#error longs and pointer have different sizes
+/* Convenience macros to test the versions of GCC.  Note - they won't
+   work for GCC1, since the _MINOR macros were not defined then, but
+   we don't have anything interesting to test for that. :-) */
+#if defined __GNUC__ && defined __GNUC_MINOR__
+# define GNUC_PREREQ(maj, min) \
+	((__GNUC__ << 16) + __GNUC_MINOR__ >= ((maj) << 16) + (min))
+#else
+# define GNUC_PREREQ(maj, min) 0
 #endif
 
-#if SIZEOF_INT < 4
-#error 16-bits platform are not supported.
+/* For internal functions, we can use the ELF hidden attribute to
+   improve code generation.  Unluckily, this is only in GCC 3.2 and
+   later */
+#ifdef HAVE_VISIBILITY_HIDDEN
+#define ATTRIBUTE_HIDDEN __attribute__ ((visibility ("hidden")))
+#else
+#define ATTRIBUTE_HIDDEN
 #endif
 
+/* At some point during the GCC 2.96 development the `pure' attribute
+   for functions was introduced.  We don't want to use it
+   unconditionally (although this would be possible) since it
+   generates warnings.
+
+   GCC 2.96 also introduced branch prediction hints for basic block
+   reordering.  We use a shorter syntax than the wordy one that GCC
+   wants.  */
+#if GNUC_PREREQ (2, 96)
+#define UNCOMMON(x) (__builtin_expect ((x) != 0, 0))
+#define COMMON(x)   (__builtin_expect ((x) != 0, 1))
+#else
+#define UNCOMMON(x) (x)
+#define COMMON(x)   (x)
+#endif
+
+/* Prefetching macros.  The NTA version is for a read that has no
+   temporal locality.  The second argument is the kind of prefetch
+   we want, using the flags that follow (T0, T1, T2, NTA follow
+   the names of the instructions in the SSE instruction set).  The
+   flags are hints, there is no guarantee that the instruction set
+   has the combination that you ask for -- just trust the compiler.
+
+   There are three macros.  PREFETCH_ADDR is for isolated prefetches,
+   for example it is used in the garbage collector's marking loop
+   to be reasonably sure that OOPs are in the cache before they're
+   marked.  PREFETCH_START and PREFETCH_LOOP usually go together, one
+   in the header of the loop and one in the middle.  However, you may
+   use PREFETCH_START only for small loops, and PREFETCH_LOOP only if
+   you know that the loop is invoked often (this is done for alloc_oop,
+   for example, to keep the next allocated OOPs in the cache).
+   PREF_BACKWARDS is for use with PREFETCH_START/PREFETCH_LOOP.  */
+#define PREF_READ 0
+#define PREF_WRITE 1
+#define PREF_BACKWARDS 2
+#define PREF_T0 0
+#define PREF_T1 4
+#define PREF_T2 8
+#define PREF_NTA 12
+
+#if GNUC_PREREQ (3, 1)
+#define DO_PREFETCH(x, distance, k) \
+  __builtin_prefetch (((char *) (x)) \
+		      + (((k) & PREF_BACKWARDS ? -(distance) : (distance)) \
+			 << L1_CACHE_SHIFT), \
+		      (k) & PREF_WRITE, \
+		      3 - (k) / (PREF_NTA / 3))
+#else
+#define DO_PREFETCH(x, distance, kind) ((void)(x))
+#endif
+
+#define PREFETCH_START(x, k) do { \
+  const char *__addr = (const char *) (x); \
+  DO_PREFETCH (__addr, 0, (k)); \
+  if (L1_CACHE_SHIFT >= 7) break; \
+  DO_PREFETCH (__addr, 1, (k)); \
+  if (L1_CACHE_SHIFT == 6) break; \
+  DO_PREFETCH (__addr, 2, (k)); \
+  DO_PREFETCH (__addr, 3, (k)); \
+} while (0)
+
+#define PREFETCH_LOOP(x, k) \
+  DO_PREFETCH ((x), (L1_CACHE_SHIFT >= 7 ? 1 : 128 >> L1_CACHE_SHIFT), (k));
+
+#define PREFETCH_ADDR(x, k) \
+  DO_PREFETCH ((x), 0, (k));
+
+/* Kill a warning when using GNU C.  Note that this allows using
+   break or continue inside a macro, unlike do...while(0) */
+#ifdef __GNUC__
+#define BEGIN_MACRO ((void) (
+#define END_MACRO ))
+#else
+#define BEGIN_MACRO if (1) 
+#define END_MACRO else (void)0
+#endif
+
+
+/* ENABLE_SECURITY enables security checks in the primitives as well as
+   special marking of untrusted objects.  Note that the code in the
+   class library to perform the security checks will be present
+   notwithstanding the setting of this flag, but they will be disabled
+   because the corresponding primitives will be made non-working.  We
+   define it here with no configure-time options because it causes
+   testsuite failures.  */
+#define ENABLE_SECURITY
 
 /* OPTIMIZE disables many checks, including consistency checks at GC
    time and bounds checking on instance variable accesses (not on #at:
@@ -51,7 +148,7 @@
    debugging painful because you know of a bug only when it's too
    late.  It is undefined because the Makefiles take care of defining
    it for optimized builds.  Bounds-checking and other errors will
-   call abort(). */
+   call abort().  */
 /* #define OPTIMIZE */
 
 typedef unsigned char gst_uchar;
@@ -66,7 +163,7 @@ typedef unsigned char gst_uchar;
 #  endif
 #endif
 
-/* If they have no const, they're likely to have no volatile, either. */
+/* If they have no const, they're likely to have no volatile, either.  */
 #ifdef const
 #define volatile
 #endif
@@ -76,29 +173,16 @@ extern char *strdup ();
 /* else it is in string.h */
 #endif
 
-/* Implementation note: on 64 bit architectures, bit-encoded 32 bit
-   words are made to live in the low order 32 bits -- the high 32 bits
-   are ignored, and are padded if necessary to achieve the useful
-   stuff in the lower 32.  This simplifies the current implementation,
-   as most bit fields, both in C and in Smalltalk do not have to be
-   modified when on a 64 bit architecture.  Smalltalk Integers will be
-   able to represent up to 62 bits + 1 sign bit. */
-#if SIZEOF_LONG == 8
-#define LONG_SHIFT 3		/* bits to shift for a long or pointer */
-#else
-#define LONG_SHIFT 2		/* bits to shift for a long or pointer */
-#endif
-
-/* Fallback to BSD functions if the standard ones are not available. */
-#ifndef HAVE_MEMCPY
-#define memcpy(s1, s2, n) bcopy((s2), (s1), (n))
-#define memcmp(s1, s2, n) bcmp ((s1), (s2), (n))
-#endif /* HAVE_MEMCPY */
-
 
 /* Run-time flags are allocated from the top, while those
    that live across image saves/restores are allocated
-   from the bottom. */
+   from the bottom.
+
+   bit 0-3: reserved for distinguishing byte objects and saving their size.
+   bit 4-11: non-volatile bits (special kinds of objects).
+   bit 12-23: volatile bits (GC/JIT-related).
+   bit 24-30: reserved for counting things.
+   bit 31: unused to avoid signedness mess. */
 enum {
   /* Place to save various OOP counts (how many fields we have marked
      in the object, how many pointer instance variables there are, 
@@ -114,29 +198,29 @@ enum {
   F_COUNT_SHIFT = 24,
 
   /* Set if the object is reachable, during the mark phases of oldspace
-     garbage collection. */
+     garbage collection.  */
   F_REACHABLE = 0x800000U,
 
   /* Set if a translation to native code is available, when running
-     with the JIT compiler enabled. */
+     with the JIT compiler enabled.  */
   F_XLAT = 0x400000U,
 
   /* Set if a translation to native code is used by the currently
-     reachable contexts. */
+     reachable contexts.  */
   F_XLAT_REACHABLE = 0x200000U,
 
   /* Set if a translation to native code is available but not used by
      the reachable contexts at the time of the last GC.  We give
      another chance to the object, but if the translation is not used
-     for two consecutive GCs we discard it. */
+     for two consecutive GCs we discard it.  */
   F_XLAT_2NDCHANCE = 0x100000U,
 
   /* Set if a translation to native code was discarded for this
      object (either because the programmer asked for this, or because
-     the method conflicted with a newly-installed method. */
+     the method conflicted with a newly-installed method).  */
   F_XLAT_DISCARDED = 0x80000U,
 
-  /* One of this is set for objects that live in newspace. */
+  /* One of this is set for objects that live in newspace.  */
   F_SPACES = 0x60000U,
   F_EVEN = 0x40000U,
   F_ODD = 0x20000U,
@@ -145,71 +229,96 @@ enum {
      in interp.c (maybe it belongs above...) */
   F_POOLED = 0x10000U,
 
-  /* The grouping of all the flags which are not valid across image
-     saves and loads. */
-  F_RUNTIME = 0xFF0000U,
+  /* Set if the bytecodes in the method have been verified. */
+  F_VERIFIED = 0x8000U,
 
-  /* Set if the OOP is currently unused. */
-  F_FREE = 0x100U,
+  /* The grouping of all the flags which are not valid across image
+     saves and loads.  */
+  F_RUNTIME = 0xFFF000U,
+
+  /* Set if the OOP is currently unused.  */
+  F_FREE = 0x10U,
 
   /* Set if the references to the instance variables of the object
-     are weak. */
-  F_WEAK = 0x200U,
+     are weak.  */
+  F_WEAK = 0x20U,
 
-  /* Set if the object is read-only. */
-  F_READONLY = 0x400U,
+  /* Set if the object is read-only.  */
+  F_READONLY = 0x40U,
 
   /* Set if the object is a context and hence its instance variables
-     are only valid up to the stack pointer. */
-  F_CONTEXT = 0x800U,
+     are only valid up to the stack pointer.  */
+  F_CONTEXT = 0x80U,
 
   /* Answer whether we want to mark the key based on references found
      outside the object.  */
-  F_EPHEMERON = 0x1000U,
+  F_EPHEMERON = 0x100U,
 
-  /* Set for objects that live in oldspace. */
-  F_OLD = 0x2000U,
+  /* Set for objects that live in oldspace.  */
+  F_OLD = 0x200U,
 
-  /* Set together with F_OLD for objects that live in fixedspace. */
-  F_FIXED = 0x4000U,
+  /* Set together with F_OLD for objects that live in fixedspace.  */
+  F_FIXED = 0x400U,
+
+  /* Set for untrusted classes, instances of untrusted classes,
+     and contexts whose receiver is untrusted.  */
+  F_UNTRUSTED = 0x800U,
 
   /* Set to the number of bytes unused in an object with byte-sized
      instance variables.  Note that this field and the following one
-     should be initialized only by INIT_BYTE_OBJECT (not really 
+     should be initialized only by INIT_UNALIGNED_OBJECT (not really 
      aesthetic but...) */
-  EMPTY_BYTES = (SIZEOF_LONG - 1),
+  EMPTY_BYTES = (sizeof (PTR) - 1),
 
   /* A bit more than what is identified by EMPTY_BYTES.  Selects some
      bits that are never zero if and only if this OOP identifies an
      object with byte instance variables.  */
-  F_BYTE = EMPTY_BYTES | SIZEOF_LONG
+  F_BYTE = 15
 };
 
-/* Answer whether an object, OOP, is readonly. */
+/* Answer whether a method, OOP, has already been verified. */
+#define IS_OOP_VERIFIED(oop) \
+  ((oop)->flags & F_VERIFIED)
+
+/* Answer whether an object, OOP, is weak.  */
 #define IS_OOP_WEAK(oop) \
-  (((oop)->flags & F_WEAK) != 0)
+  ((oop)->flags & F_WEAK)
 
-/* Answer whether an object, OOP, is readonly. */
+/* Answer whether an object, OOP, is readonly.  */
 #define IS_OOP_READONLY(oop) \
-  (IS_INT((oop)) ? true : ((oop)->flags & F_READONLY) != 0)
+  (IS_INT ((oop)) ? F_READONLY : (oop)->flags & F_READONLY)
 
-/* Set whether an object, OOP, is readonly or readwrite. */
-#define MAKE_OOP_READONLY(oop, ro) do { \
-  if (ro) { (oop)->flags |= F_READONLY; } \
-  else { (oop)->flags &= ~F_READONLY; } \
-} while(0)
+/* Set whether an object, OOP, is readonly or readwrite.  */
+#define MAKE_OOP_READONLY(oop, ro) \
+  (((oop)->flags &= ~F_READONLY), \
+   ((oop)->flags |= (ro) ? F_READONLY : 0))
 
-/* Set whether an object, OOP, has a finalizer. */
-#define MAKE_OOP_EPHEMERON(oop) do { \
-  (oop)->flags |= F_EPHEMERON; \
-} while(0)
+#ifdef ENABLE_SECURITY
+
+/* Answer whether an object, OOP, is untrusted.  */
+#define IS_OOP_UNTRUSTED(oop) \
+  (IS_INT ((oop)) ? 0 : ((oop)->flags & F_UNTRUSTED))
+
+/* Set whether an object, OOP, is trusted or untrusted.  */
+#define MAKE_OOP_UNTRUSTED(oop, untr) \
+  (((oop)->flags &= ~F_UNTRUSTED), \
+   ((oop)->flags |= (untr) ? F_UNTRUSTED : 0))
+
+#else
+#define IS_OOP_UNTRUSTED(oop) (false)
+#define MAKE_OOP_UNTRUSTED(oop, untr) ((void)0)
+#endif
+
+/* Set whether an object, OOP, has ephemeron semantics.  */
+#define MAKE_OOP_EPHEMERON(oop) \
+  (oop)->flags |= F_EPHEMERON;
 
 
 /*
       3                   2                   1 
     1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |    # fixed fields             |      unused         |p|w|F|0|1|
+   |    # fixed fields             |      unused       |I| kind  |1|
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
    I'm moving it to bits 13-30 (it used to stay at bits 5-30),
@@ -218,41 +327,67 @@ enum {
    Remember to shift by ISP_NUMFIXEDFIELDS-1 there, since Smalltalk does
    not see ISP_INTMARK!!
 
-   FIXME: these should be exported in a pool dictionary. */
+   Keep these in sync with _gst_sizes, in dict.c.
+
+   FIXME: these should be exported in a pool dictionary.  */
 
 enum {
-  /* This is a shift count. */
+  /* This is a shift count.  */
   ISP_NUMFIXEDFIELDS = 13,
 
-  /* These represent the shape of the indexed instance variables of
-     the instances of the class. */
-  ISP_INDEXEDVARS = 28,         
+  /* Set if the indexed instance variables are pointers.  */
+  ISP_FIXED = 0,
+  ISP_SCHAR = 32,
+  ISP_UCHAR = 34,
+  ISP_SHORT = 36,
+  ISP_USHORT = 38,
+  ISP_INT = 40,
+  ISP_UINT = 42,
+  ISP_FLOAT = 44,
+  ISP_INT64 = 46,
+  ISP_UINT64 = 48,
+  ISP_DOUBLE = 50,
+  ISP_LAST_SCALAR = 50,
+  ISP_POINTER = 62,
 
-  /* Set if the indexed instance variables are pointers. */
-  ISP_ISPOINTERS = 16,
-
-  /* Set if the indexed instance variables are words (C longs). */
-  ISP_ISWORDS = 8,
+#if SIZEOF_OOP == 8
+  ISP_LONG = ISP_INT64,
+  ISP_ULONG = ISP_UINT64,
+  ISP_LAST_UNALIGNED = ISP_FLOAT,
+#else
+  ISP_LONG = ISP_INT,
+  ISP_ULONG = ISP_UINT,
+  ISP_LAST_UNALIGNED = ISP_USHORT,
+#endif
 
   /* Set if the instances of the class have indexed instance
-     variables. */
-  ISP_ISINDEXABLE = 4,
+     variables.  */
+  ISP_ISINDEXABLE = 32,
 
-  /* Set to 1 to mark a SmallInteger. */
+  /* These represent the shape of the indexed instance variables of
+     the instances of the class.  */
+  ISP_INDEXEDVARS = 62,
+  ISP_SHAPE = 30,
+
+  /* Set to 1 to mark a SmallInteger.  */
   ISP_INTMARK = 1
 };
 
 
 /* the current execution stack pointer */
-#ifndef USE_JIT_TRANSLATION
+#ifndef ENABLE_JIT_TRANSLATION
 # define sp		_gst_sp
 #endif
 
 /* The VM's stack pointer */
-extern OOP *sp;
+extern OOP *sp 
+  ATTRIBUTE_HIDDEN;
 
 /* Some useful constants */
-extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
+extern OOP _gst_nil_oop 
+  ATTRIBUTE_HIDDEN, _gst_true_oop 
+  ATTRIBUTE_HIDDEN, _gst_false_oop 
+  ATTRIBUTE_HIDDEN;
 
 /* Some stack operations */
 #define UNCHECKED_PUSH_OOP(oop) \
@@ -273,7 +408,7 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #define PUSH_OOP(oop) \
   do { \
     OOP __pushOOP = (oop); \
-    UNCHECKED_PUSH_OOP(__pushOOP); \
+    UNCHECKED_PUSH_OOP (__pushOOP); \
   } while (0)
 #endif
 
@@ -325,11 +460,17 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 
 
 /* Answer whether CLASS is the class that the object pointed to by OOP
-   belongs to.  OOP can also be a SmallInteger. */
+   belongs to.  OOP can also be a SmallInteger.  */
 #define IS_CLASS(oop, class) \
-  (IS_INT(oop) ? class == _gst_small_integer_class : class == OOP_CLASS(oop))
+  (OOP_INT_CLASS(oop) == class)
 
-/* Answer whether OOP is nil. */
+/* Answer the CLASS that the object pointed to by OOP belongs to.  OOP
+   can also be a SmallInteger. */
+#define OOP_INT_CLASS(oop) \
+  (IS_INT(oop) ? _gst_small_integer_class : OOP_CLASS(oop))
+
+
+/* Answer whether OOP is nil.  */
 #define IS_NIL(oop) \
   ((OOP)(oop) == _gst_nil_oop)
 
@@ -338,65 +479,65 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
    emptyBytes field is guaranteed to be zero.
 
    Note that F_BYTE is a bit more than EMPTY_BYTES, so that if value 
-   is a multiple of SIZEOF_LONG the flags identified by F_BYTE are
+   is a multiple of sizeof (PTR) the flags identified by F_BYTE are
    not zero.  */
-#define INIT_BYTE_OBJECT(oop, value) \
-    ((oop)->flags |= SIZEOF_LONG - ((value) & EMPTY_BYTES))
+#define INIT_UNALIGNED_OBJECT(oop, value) \
+    ((oop)->flags |= sizeof (PTR) | (value))
 
 
 /* Generally useful conversion functions */
 #define SIZE_TO_BYTES(size) \
-  ((size) << LONG_SHIFT)
+  ((size) * sizeof (PTR))
 
 #define BYTES_TO_SIZE(bytes) \
-  ((bytes) >> LONG_SHIFT)
+  ((bytes) / sizeof (PTR))
 
 
-/* integer conversions and some information on SmallIntegers. */
+/* integer conversions and some information on SmallIntegers.  */
 
 #define TO_INT(oop) \
-  ((long)(oop) >> 1)
+  ((intptr_t)(oop) >> 1)
 
 #define FROM_INT(i) \
-  (OOP)( ((long)(i) << 1) | 1)
+  (OOP)( ((intptr_t)(i) << 1) + 1)
 
-#define ST_INT_SIZE        ((SIZEOF_LONG * 8) - 2)
+#define ST_INT_SIZE        ((sizeof (PTR) * 8) - 2)
 #define MAX_ST_INT         ((1L << ST_INT_SIZE) - 1)
 #define MIN_ST_INT         ( ~MAX_ST_INT)
 #define INT_OVERFLOW(i)    ( (i) > MAX_ST_INT || (i) < MIN_ST_INT )
 #define OVERFLOWING_INT    (MAX_ST_INT + 1)
 
-#define INCR_INT(i)         ((OOP) (((long)i) + 2))	/* 1 << 1 */
-#define DECR_INT(i)         ((OOP) (((long)i) - 2))	/* 1 << 1 */
+#define INCR_INT(i)         ((OOP) (((intptr_t)i) + 2))	/* 1 << 1 */
+#define DECR_INT(i)         ((OOP) (((intptr_t)i) - 2))	/* 1 << 1 */
 
 /* Endian conversions, using networking functions if they do
    the correct job (that is, on 32-bit little-endian systems)
-   because they are likely to be optimized. */
+   because they are likely to be optimized.  */
 
-#if SIZEOF_LONG == 4
+#if SIZEOF_OOP == 4
 # if defined(WORDS_BIGENDIAN) || !defined (HAVE_INET_SOCKETS)
 #  define BYTE_INVERT(x) \
-        ((unsigned long int)((((unsigned long int)(x) & 0x000000ffU) << 24) | \
-                             (((unsigned long int)(x) & 0x0000ff00U) <<  8) | \
-                             (((unsigned long int)(x) & 0x00ff0000U) >>  8) | \
-                             (((unsigned long int)(x) & 0xff000000U) >> 24)))
+        ((uintptr_t)((((uintptr_t)(x) & 0x000000ffU) << 24) | \
+                     (((uintptr_t)(x) & 0x0000ff00U) <<  8) | \
+                     (((uintptr_t)(x) & 0x00ff0000U) >>  8) | \
+                     (((uintptr_t)(x) & 0xff000000U) >> 24)))
 # else
 #  define BYTE_INVERT(x) htonl((x))
 # endif
 
-#else /* SIZEOF_LONG == 8 */
+#else /* SIZEOF_OOP == 8 */
 # define BYTE_INVERT(x) \
-        ((unsigned long)((((unsigned long)(x) & 0x00000000000000ffU) << 56) | \
-                         (((unsigned long)(x) & 0x000000000000ff00U) << 40) | \
-                         (((unsigned long)(x) & 0x0000000000ff0000U) << 24) | \
-                         (((unsigned long)(x) & 0x00000000ff000000U) <<  8) | \
-                         (((unsigned long)(x) & 0x000000ff00000000U) >>  8) | \
-                         (((unsigned long)(x) & 0x0000ff0000000000U) >> 24) | \
-                         (((unsigned long)(x) & 0x00ff000000000000U) >> 40) | \
-                         (((unsigned long)(x) & 0xff00000000000000U) >> 56)))
-#endif /* SIZEOF_LONG == 8 */
+        ((uintptr_t)((((uintptr_t)(x) & 0x00000000000000ffU) << 56) | \
+                     (((uintptr_t)(x) & 0x000000000000ff00U) << 40) | \
+                     (((uintptr_t)(x) & 0x0000000000ff0000U) << 24) | \
+                     (((uintptr_t)(x) & 0x00000000ff000000U) <<  8) | \
+                     (((uintptr_t)(x) & 0x000000ff00000000U) >>  8) | \
+                     (((uintptr_t)(x) & 0x0000ff0000000000U) >> 24) | \
+                     (((uintptr_t)(x) & 0x00ff000000000000U) >> 40) | \
+                     (((uintptr_t)(x) & 0xff00000000000000U) >> 56)))
+#endif /* SIZEOF_OOP == 8 */
 
-/* The standard min/max macros... */
+/* The standard min/max macros...  */
 
 #ifndef ABS
 #define ABS(x) (x >= 0 ? x : -x)
@@ -408,42 +549,6 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #define MIN(x, y) 		( ((x) > (y)) ? (y) : (x) )
 #endif
 
-/* Convenience macros to test the versions of GCC.  Note - they won't
-   work for GCC1, since the _MINOR macros were not defined then, but
-   we don't have anything interesting to test for that. :-) */
-#if defined __GNUC__ && defined __GNUC_MINOR__
-# define GNUC_PREREQ(maj, min) \
-	((__GNUC__ << 16) + __GNUC_MINOR__ >= ((maj) << 16) + (min))
-#else
-# define GNUC_PREREQ(maj, min) 0
-#endif
-
-/* At some point during the GCC 2.96 development the `pure' attribute
-   for functions was introduced.  We don't want to use it
-   unconditionally (although this would be possible) since it
-   generates warnings.
-
-   GCC 2.96 also introduced branch prediction hints for basic block
-   reordering.  We use a shorter syntax than the wordy one that GCC
-   wants. */
-#if GNUC_PREREQ (2, 96)
-#define UNCOMMON(x) (__builtin_expect ((x) != 0, 0))
-#define COMMON(x)   (__builtin_expect ((x) != 0, 1))
-#else
-#define UNCOMMON(x) (x)
-#define COMMON(x)   (x)
-#endif
-
-/* Kill a warning when using GNU C.  Note that this allows using
-   break or continue inside a macro, unlike do...while(0) */
-#ifdef __GNUC__
-#define BEGIN_MACRO ((void) (
-#define END_MACRO ))
-#else
-#define BEGIN_MACRO if (1) 
-#define END_MACRO else (void)0
-#endif
-
 #include <sys/types.h>
 #include <stdio.h>
 #include <stddef.h>
@@ -506,25 +611,7 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #include <sys/mman.h>
 #endif
 
-/* TODO: replace this cruft with autoconf tests that generate a stdint.h */
-#if defined HAVE_INTTYPES_H
-#include <inttypes.h>
-#elif defined HAVE_STDINT_H
-#include <stdint.h>
-#elif defined __GNUC__
-typedef long long int64_t;
-typedef unsigned long long uint64_t;
-#endif
-
-#if defined HAVE_INTTYPES_H || defined HAVE_STDINT_H
-#ifndef UINTMAX_C
-#define UINTMAX_C(n) n##UL
-#endif
-#ifndef UINTMAX_MAX
-#define UINTMAX_MAX ULONG_MAX
-#endif
-#endif
-
+#include "stdintx.h"
 #include "ansidecl.h"
 #include "signalx.h"
 #include "mathl.h"
@@ -538,18 +625,19 @@ typedef unsigned long long uint64_t;
 
 #include "tree.h"
 #include "input.h"
-#include "byte.h"
 #include "callin.h"
 #include "cint.h"
 #include "dict.h"
 #include "events.h"
 #include "gstpub.h"
 #include "heap.h"
-#include "comp.h"
-#include "interp.h"
 #include "lex.h"
+#include "gst-parse.h"
 #include "lib.h"
 #include "oop.h"
+#include "byte.h"
+#include "comp.h"
+#include "interp.h"
 #include "opt.h"
 #include "save.h"
 #include "str.h"
@@ -558,6 +646,8 @@ typedef unsigned long long uint64_t;
 #include "xlat.h"
 #include "mpz.h"
 #include "print.h"
+#include "security.h"
+#include "match.h"
 
 /* Include this last, it has the bad habit of #defining printf
    and this fools gcc's __attribute__ (format) */
