@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 2001, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright 2001, 2002, 2003, 2005 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -66,12 +66,22 @@ oop_stream;
 typedef struct input_stream
 {
   stream_type type;
-  char pushedBackChars[3];	/* the 3 buffered characters */
-  int pushedBackCount;		/* number of chars pushed back */
+  /* the 3 buffered characters */
+  char pushedBackChars[3];
+
+  /* number of chars pushed back */
+  int pushedBackCount;
+
   int line;
   int column;
   const char *fileName;
   int fileOffset;
+
+  /* The starting-position state.  If this is <0, as soon as another
+     token is lexed, the flag is reinitialized to the starting position
+     of the next token.  */
+  off_t method_start_pos;
+
   OOP fileNameOOP;		/* the full path name for file */
   mst_Boolean prompt;
   union
@@ -89,37 +99,34 @@ typedef struct input_stream
 #define st_str		st.u_st_str
 #define st_oop		st.u_st_oop
 
-/* The internal interface used by _gst_next_char. */
+/* The internal interface used by _gst_next_char.  */
 static int my_getc (input_stream stream);
 
-/* Print a line indicator in front of an error message. */
+/* Print a line indicator in front of an error message.  */
 static void line_stamp (int line);
 
-/* Close the given stream */
-static void my_close (input_stream stream);
-
 /* Allocate and push a new stream of type TYPE on the stack; the new
-   stream is then available through IN_STREAM. */
+   stream is then available through IN_STREAM.  */
 static input_stream push_new_stream (stream_type type);
 
 /* The topmost stream in the stack, and the head of the linked list
-   that implements the stack. */
+   that implements the stack.  */
 static input_stream in_stream = NULL;
 
-/* Controls the use of the changes file, for recording source text.
-   If NULL, no recording */
-char *_gst_change_file_name = NULL;
+/* Poll FD until it is available for input (or until it returns
+   POLLHUP) and then perform a read system call.  */
+static int poll_and_read (int fd, char *buf, int n);
 
 static int change_str = -1;
 
 
 /* If true, the normal execution information is supressed, and the
    prompt is emitted with a special marker character ahead of it to
-   let the process filter know that the execution has completed. */
+   let the process filter know that the execution has completed.  */
 mst_Boolean _gst_emacs_process = false;
 
 /* >= 1 if completions are enabled, < 1 if they are not.  Available
-   for completeness even if Readline is not used. */
+   for completeness even if Readline is not used.  */
 static int completions_enabled = 1;
 
 
@@ -127,14 +134,14 @@ static int completions_enabled = 1;
 /* Storage for the possible completions */
 static char **completions;
 
-/* Number of completions available. */
+/* Number of completions available.  */
 static int count;
 
-/* Number of completions before the array must be resized. */
+/* Number of completions before the array must be resized.  */
 static int free_items;
 
 /* Number of sorted completions.  Completions are not sorted until
-   we are requested to use them. */
+   we are requested to use them.  */
 static int sorted_count;
 
 /* Internal functions */
@@ -176,21 +183,48 @@ static char **readline_match_symbols (char *text,
    The `readline()' interface: The behavior is like the Smalltalk
    String interface.  The end-of-string or a NULL strBase-pointer
    decides to read in a new line.  The prompt is still shown by the
-   readline() call. */
+   readline() call.  */
 
 void
 _gst_pop_stream (mst_Boolean closeIt)
 {
-  input_stream oldStream;
+  input_stream stream;
 
-  oldStream = in_stream;
+  stream = in_stream;
   in_stream = in_stream->prevStream;
 
-  if (closeIt && oldStream)
+  if (!IS_NIL (stream->fileNameOOP))
+    _gst_unregister_oop (stream->fileNameOOP);
+
+  switch (stream->type)
     {
-      my_close (oldStream);
-      xfree (oldStream);
+    case STREAM_STRING:
+      xfree (stream->st_str.strBase);
+      break;
+
+    case STREAM_OOP:
+      xfree (stream->st_oop.buf);
+      _gst_unregister_oop (stream->st_oop.oop);
+      break;
+
+    case STREAM_FILE:
+      xfree (stream->st_file.buf);
+      if (closeIt)
+        close (stream->st_file.fd);
+      break;
+
+#ifdef HAVE_READLINE
+    case STREAM_READLINE:
+      if (stream->st_str.strBase)
+	{
+	  xfree (stream->st_str.strBase);
+	  stream->st_str.strBase = NULL;
+	}
+      break;
+#endif /* HAVE_READLINE */
     }
+
+  xfree (stream);
 }
 
 void
@@ -218,9 +252,15 @@ _gst_push_stream_oop (OOP oop)
   newStream->st_oop.buf = NULL;
   newStream->st_oop.ptr = NULL;
   newStream->st_oop.end = NULL;
-  newStream->fileName = "a Smalltalk Stream";
-  newStream->prompt = false;
+  if (is_a_kind_of (OOP_CLASS (oop), _gst_file_descriptor_class))
+    {
+      newStream->fileNameOOP = ((gst_file_stream) OOP_TO_OBJ (oop))->name;
+      _gst_register_oop (newStream->fileNameOOP);
+    }
+  else
+    newStream->fileName = "a Smalltalk Stream";
 
+  newStream->prompt = false;
   _gst_register_oop (oop);
 }
 
@@ -285,10 +325,12 @@ push_new_stream (stream_type type)
   newStream->column = 0;
   newStream->fileOffset = 0;
   newStream->type = type;
+  newStream->fileName = NULL;
   newStream->fileNameOOP = _gst_nil_oop;
   newStream->prevStream = in_stream;
   in_stream = newStream;
 
+  _gst_clear_method_start_pos ();
   return (newStream);
 }
 
@@ -300,11 +342,25 @@ _gst_set_stream_info (int line,
 {
   in_stream->line = line;
   in_stream->column = 0;
-  if (in_stream->type == STREAM_FILE)
+  _gst_clear_method_start_pos ();
+  if (fileName)
     {
       in_stream->fileName = fileName;
-      in_stream->fileOffset = fileOffset;
+      in_stream->fileNameOOP = _gst_nil_oop;
     }
+  in_stream->fileOffset = fileOffset;
+}
+ 
+off_t
+_gst_get_method_start_pos (void)
+{
+  return (in_stream->method_start_pos);
+}
+ 
+void
+_gst_clear_method_start_pos (void)
+{
+  in_stream->method_start_pos = -1;
 }
 
 int
@@ -324,21 +380,19 @@ my_getc (input_stream stream)
       return ic;
 
     case STREAM_OOP:
-      /* Refill the buffer... */
+      /* Refill the buffer...  */
       if (stream->st_oop.ptr == stream->st_oop.end)
 	{
 	  if (stream->st_oop.buf)
-	    {
-	      xfree (stream->st_oop.buf);
-	      stream->st_oop.buf = NULL;
-	    }
+	    xfree (stream->st_oop.buf);
 
-	  _gst_msg_sendf(stream->st_oop.buf, "%s %o nextHunk",
+	  _gst_msg_sendf(&stream->st_oop.buf, "%s %o nextHunk",
 			 stream->st_oop.oop);
 
-	  stream->st_oop.end = stream->st_oop.buf +
-	    strlen (stream->st_oop.buf);
 	  stream->st_oop.ptr = stream->st_oop.buf;
+	  stream->st_oop.end = stream->st_oop.buf;
+	  if (stream->st_oop.buf)
+	    stream->st_oop.end += strlen (stream->st_oop.buf);
 	}
 
       return (stream->st_oop.ptr == stream->st_oop.end)
@@ -354,16 +408,10 @@ my_getc (input_stream stream)
 	  fflush(stdout);
 	}
 
-      /* Refill the buffer... */
+      /* Refill the buffer...  */
       if (stream->st_file.ptr == stream->st_file.end)
 	{
-	  int n;
-	  do
-	    {
-	      errno = 0;
-	      n = _gst_read (stream->st_file.fd, stream->st_file.buf, 1024);
-	    }
-	  while ((n == -1) && (errno == EINTR));
+	  int n = poll_and_read (stream->st_file.fd, stream->st_file.buf, 1024);
 	  if (n < 0)
 	    n = 0;
 
@@ -394,7 +442,7 @@ my_getc (input_stream stream)
 	    xfree (stream->st_str.strBase);
 	    stream->st_str.strBase = NULL;
 	  }
-	r_line = readline ("st> ");
+	r_line = readline ((char *) "st> ");
 	if (!r_line)
 	  {
 	    /* return value of NULL indicates EOF */
@@ -402,7 +450,7 @@ my_getc (input_stream stream)
 	  }
 	if (*r_line)
 	  {
-	    /* add only non-empty lines. */
+	    /* add only non-empty lines.  */
 	    add_history (r_line);
 	  }
 
@@ -413,7 +461,7 @@ my_getc (input_stream stream)
 	  {
 	    _gst_errorf ("Out of memory reallocating linebuffer space");
 	    stream->st_str.str = stream->st_str.strBase = NULL;
-	    ic = '\n';		/* return a newline ... */
+	    ic = '\n';		/* return a newline ...  */
 	  }
 	else
 	  {
@@ -434,44 +482,6 @@ my_getc (input_stream stream)
   return (ic);
 }
 
-void
-my_close (input_stream stream)
-{
-  if (!IS_NIL (stream->fileNameOOP))
-    _gst_unregister_oop (stream->fileNameOOP);
-
-  switch (stream->type)
-    {
-    case STREAM_STRING:
-      xfree (stream->st_str.strBase);
-      break;
-
-    case STREAM_OOP:
-      xfree (stream->st_oop.buf);
-      _gst_unregister_oop (stream->st_oop.oop);
-      break;
-
-    case STREAM_FILE:
-      xfree (stream->st_file.buf);
-      close (stream->st_file.fd);
-      break;
-
-#ifdef HAVE_READLINE
-    case STREAM_READLINE:
-      if (stream->st_str.strBase)
-	{
-	  xfree (stream->st_str.strBase);
-	  stream->st_str.strBase = NULL;
-	}
-      break;
-#endif /* HAVE_READLINE */
-
-    default:
-      _gst_errorf ("Bad stream type passed to my_close");
-      _gst_had_error = true;
-    }
-}
-
 
 mst_Boolean
 _gst_get_cur_stream_prompt (void)
@@ -642,6 +652,10 @@ _gst_get_location (int *x, int *y)
 {
   *x = in_stream->column;
   *y = in_stream->line;
+
+  /* Subtract 1 to mark the position of the last character we read.  */
+  if (in_stream->method_start_pos < 0)
+    in_stream->method_start_pos = _gst_get_cur_file_pos () - 1;
 }
 
 void
@@ -656,6 +670,11 @@ line_stamp (int line)
 	{
 	  if (in_stream->fileName)
 	    fprintf (stderr, "%s:", in_stream->fileName);
+	  else if (in_stream->fileNameOOP != _gst_nil_oop)
+	    {
+	      char *s = _gst_to_cstring (in_stream->fileNameOOP);
+	      fprintf (stderr, "%s:", s);
+	    }
 
 	  fprintf (stderr, "%d: ", line);
 	}
@@ -684,8 +703,7 @@ line_stamp (int line)
 }
 
 
-
-int
+off_t
 _gst_get_cur_file_pos (void)
 {
   /* ### not sure about this particular use -- the current file pos
@@ -710,25 +728,6 @@ _gst_get_cur_file_pos (void)
     return (-1);
 }
 
-void
-_gst_init_changes_stream (void)
-{
-  if (_gst_change_file_name)
-    change_str = _gst_open_file (_gst_change_file_name, "a");
-}
-
-void
-_gst_reset_changes_file (void)
-{
-  if (!_gst_change_file_name)
-    return;
-  if (change_str != -1)
-    close (change_str);
-
-  remove (_gst_change_file_name);
-  if (change_str != -1)
-    _gst_init_changes_stream ();
-}
 
 
 int
@@ -738,7 +737,8 @@ _gst_next_char (void)
 
   if (in_stream->pushedBackCount > 0)
     {
-      ic = in_stream->pushedBackChars[--in_stream->pushedBackCount];
+      ic = (unsigned char)
+	      in_stream->pushedBackChars[--in_stream->pushedBackCount];
       return (ic);
     }
   else
@@ -768,7 +768,8 @@ _gst_unread_char (int ic)
 }
 
 
-/* These two are not used, but are provided for additional flexibility. */
+
+/* These two are not used, but are provided for additional flexibility.  */
 void
 _gst_enable_completion (void)
 {
@@ -781,6 +782,38 @@ _gst_disable_completion (void)
   completions_enabled--;
 }
 
+int
+poll_and_read (int fd, char *buf, int n)
+{
+  int result;
+  struct pollfd pfd;
+
+  pfd.fd = fd;
+  pfd.events = POLLIN;
+  pfd.revents = 0;
+
+  do
+    {
+      errno = 0;
+      result = poll (&pfd, 1, -1); /* Infinite wait */
+    }
+  while ((result == 0 && (pfd.revents & POLLHUP) == 0)
+	 || ((result == -1) && (errno == EINTR)));
+
+  if (pfd.revents & POLLIN)
+    {
+      do
+	{
+	  errno = 0;
+	  result = _gst_read (fd, buf, n);
+	}
+      while ((result == -1) && (errno == EINTR));
+      return result;
+    }
+  else
+    return -1;
+}
+
 #ifdef HAVE_READLINE
 /* Find apostrophes and double them */
 char *
@@ -846,7 +879,7 @@ add_completion (const char *str,
     {
       free_items += 50;
       completions =
-	(char **) xrealloc (completions, SIZEOF_CHAR_P * (count + 50));
+	(char **) xrealloc (completions, sizeof (char *) * (count + 50));
     }
 
   free_items--;
@@ -896,14 +929,14 @@ merge (char **a1,
 
   if (!count2)
     {
-      memmove (a1, a2, count1 * SIZEOF_CHAR_P);
+      memmove (a1, a2, count1 * sizeof (char *));
       return;
     }
 
   if (reallocate)
     {
-      char **new = (char **) alloca (count1 * SIZEOF_CHAR_P);
-      memcpy (new, a1, count1 * SIZEOF_CHAR_P);
+      char **new = (char **) alloca (count1 * sizeof (char *));
+      memcpy (new, a1, count1 * sizeof (char *));
       a1 = new;
     }
 
@@ -918,7 +951,7 @@ merge (char **a1,
 	  if (--count2 == 0)
 	    {
 	      /* Any leftovers from the source array? */
-	      memcpy (a1, a2, count1 * SIZEOF_CHAR_P);
+	      memcpy (a1, a2, count1 * sizeof (char *));
 	      return;
 	    }
 
@@ -959,7 +992,7 @@ symbol_generator (const char *text,
     return (NULL);
 
   /* Since we have to sort the array to perform the binary search, we
-     remove duplicates and avoid that readline resorts the result. */
+     remove duplicates and avoid that readline resorts the result.  */
   while (matches_left > 1 &&
 	 strcmp (completions[current_index],
 		 completions[current_index + 1]) == 0)
@@ -991,11 +1024,11 @@ readline_match_symbols (char *text,
 
   /* Prepare for binary searching.  We use qsort when necessary, and
      merge the result, instead of doing expensive (quadratic) insertion
-     sorts. */
+     sorts.  */
   if (sorted_count < count)
     {
       qsort (&completions[sorted_count], count - sorted_count,
-	     SIZEOF_CHAR_P, compare_strings);
+	     sizeof (char *), compare_strings);
 
       merge (&completions[sorted_count], count - sorted_count,
 	     completions, sorted_count, true);
@@ -1004,10 +1037,10 @@ readline_match_symbols (char *text,
     }
 
   /* Initialize current_index and matches_left with two binary
-     searches. */
+     searches.  */
   len = strlen (text);
 
-  /* The first binary search gives the first matching item. */
+  /* The first binary search gives the first matching item.  */
   low = -1;
   high = count;
   while (low + 1 != high)
@@ -1042,31 +1075,9 @@ readline_getc (FILE * file)
 {
   int result;
   unsigned char ch;
-  struct pollfd pfd;
+  result = poll_and_read (fileno (file), &ch, 1);
 
-  pfd.fd = fileno (file);
-  pfd.events = POLLIN;
-  pfd.revents = 0;
-
-  do
-    {
-      errno = 0;
-      result = poll (&pfd, 1, -1); /* Infinite wait */
-    }
-  while ((result == -1) && (errno == EINTR));
-
-  if (pfd.revents & POLLIN)
-    {
-      do
-	{
-	  errno = 0;
-	  result = _gst_read (fileno (file), &ch, 1);
-	}
-      while ((result == -1) && (errno == EINTR));
-      return (result < 1) ? EOF : (int) ch;
-    }
-  else
-    return EOF;
+  return (result < 1) ? EOF : (int) ch;
 }
 
 void
@@ -1075,20 +1086,20 @@ _gst_initialize_readline (void)
   static char everything[255];
   int i;
 
-  /* Allow conditional parsing of the ~/.inputrc file. */
-  rl_readline_name = "Smalltalk";
+  /* Allow conditional parsing of the ~/.inputrc file.  */
+  rl_readline_name = (char *) "Smalltalk";
 
   /* Always put filenames in quotes */
   for (i = 0; i < 255; i++)
     everything[i] = i + 1;
 
   rl_filename_quote_characters = everything;
-  rl_completer_quote_characters = "'\"";
-  rl_basic_word_break_characters = "() []{};+-=*<>~'?%/@|&#^\"\\.";
+  rl_completer_quote_characters = (char *) "'\"";
+  rl_basic_word_break_characters = (char *) "() []{};+-=*<>~'?%/@|&#^\"\\.";
 
   /* Consider binary selectors both word-breaking characters and
      candidates for completion */
-  rl_special_prefixes = "+-=*<>~?%/@|&\\";
+  rl_special_prefixes = (char *) "+-=*<>~?%/@|&\\";
 
   /* Our rules for quoting are a bit different from the default */
   rl_filename_quoting_function = (CPFunction *) readline_quote_filename;
@@ -1100,7 +1111,7 @@ _gst_initialize_readline (void)
     (CPPFunction *) readline_match_symbols;
 
   /* Since we have to sort the array to perform the binary search,
-     remove duplicates and avoid that readline resorts the result. */
+     remove duplicates and avoid that readline resorts the result.  */
   rl_ignore_completion_duplicates = 0;
 
   /* Set up to use read to read from stdin */
