@@ -46,133 +46,172 @@ stream_type;
 
 /* If true, the normal execution information is supressed, and the
    prompt is emitted with a special marker character ahead of it to
-   let the process filter know that the execution has completed. */
-extern mst_Boolean _gst_emacs_process;
+   let the process filter know that the execution has completed.  */
+extern mst_Boolean _gst_emacs_process 
+  ATTRIBUTE_HIDDEN;
 
 /* Pass file descriptor FD, printed as file name FILENAME, to the
-   parser. */
+   parser.  */
 extern void _gst_push_unix_file (int fd,
-				 const char *fileName);
+				 const char *fileName) 
+  ATTRIBUTE_HIDDEN;
 
-/* Return the current line and column into Y and X, respectively.  */
-extern void _gst_get_location (int *x, int *y);
+/* Return the position in the stream where the method had started.  */
+extern off_t _gst_get_method_start_pos (void)
+  ATTRIBUTE_PURE
+  ATTRIBUTE_HIDDEN; 
+ 
+/* Clear the starting-position state.  As soon as another token is
+   lexed, the flag is reinitialized to the starting position of the
+   next token.  */
+extern void _gst_clear_method_start_pos (void) 
+  ATTRIBUTE_HIDDEN;
+ 
+/* Return the current line and column into Y and X, respectively.  
+   Also sets the method_start_pos pointer.  */
+extern void _gst_get_location (int *x, int *y) 
+  ATTRIBUTE_HIDDEN;
 
-/* Pass the OOP to the parser; it must respond to #nextHunk. */
-extern void _gst_push_stream_oop (OOP oop);
+/* Pass the OOP to the parser; it must respond to #nextHunk.  */
+extern void _gst_push_stream_oop (OOP oop) 
+  ATTRIBUTE_HIDDEN;
 
 /* Pass the contents of the Smalltalk String STRINGOOP to the parser.
    While the String is being parsed, it can change because it is
-   saved.  The String should not contain nulls. */
-extern void _gst_push_smalltalk_string (OOP stringOOP);
+   saved.  The String should not contain nulls.  */
+extern void _gst_push_smalltalk_string (OOP stringOOP) 
+  ATTRIBUTE_HIDDEN;
 
 /* Pass the contents of the null-terminated String STRING to the
    parser.  While the String is being parsed, it can change because it
-   is saved. */
-extern void _gst_push_cstring (const char *string);
+   is saved.  */
+extern void _gst_push_cstring (const char *string) 
+  ATTRIBUTE_HIDDEN;
 
 /* Pass the contents of stdin (either through the readline interface
    or as a vanilla file descriptor) to the parser.  The readline
    interface is used if the Emacs interface is not active and stdin is
-   a tty. */
-extern void _gst_push_stdin_string (void);
+   a tty.  */
+extern void _gst_push_stdin_string (void) 
+  ATTRIBUTE_HIDDEN;
 
 /* Restores the previous stream on the stack, optionally closing the
-   topmost one if CLOSEIT is true. */
-extern void _gst_pop_stream (mst_Boolean closeIt);
+   topmost one if CLOSEIT is true.  */
+extern void _gst_pop_stream (mst_Boolean closeIt) 
+  ATTRIBUTE_HIDDEN;
 
 /* This function resets the file type information for the current
    stream.  It is typically used by fileIn type methods when filing in
    a subsection of a real file via a temporary file what the real
-   source of the text is. */
+   source of the text is.  */
 extern void _gst_set_stream_info (int line,
 				  const char *fileName,
-				  int fileOffset);
-
-/* Opens the changelog for append. */
-extern void _gst_init_changes_stream (void);
-
-/* Resets the changes file to be zero length. */
-extern void _gst_reset_changes_file (void);
+				  int fileOffset) 
+  ATTRIBUTE_HIDDEN;
 
 /* Emits a warning, with the current file and line in front of it, on
-   the standard error descriptor. */
+   the standard error descriptor.  */
 extern void _gst_warningf (const char *str,
-			   ...) ATTRIBUTE_PRINTF_1;
+			   ...)
+  ATTRIBUTE_PRINTF_1 
+  ATTRIBUTE_HIDDEN;
 
 /* Emits a warning, with the current file and the given LINE in front
-   of it, on the standard error descriptor. */
+   of it, on the standard error descriptor.  */
 extern void _gst_warningf_at (int line,
 			      const char *str,
-			      ...) ATTRIBUTE_PRINTF_2;
+			      ...)
+  ATTRIBUTE_PRINTF_2 
+  ATTRIBUTE_HIDDEN;
 
 /* Emits an error, with the current file and line in front of it, on
-   the standard error descriptor. */
+   the standard error descriptor.  */
 extern void _gst_errorf (const char *str,
-			 ...) ATTRIBUTE_PRINTF_1;
+			 ...)
+  ATTRIBUTE_PRINTF_1 
+  ATTRIBUTE_HIDDEN;
 
 /* Emits an error, with the current file and the given LINE in front
-   of it, on the standard error descriptor. */
+   of it, on the standard error descriptor.  */
 extern void _gst_errorf_at (int line,
 			    const char *str,
-			    ...) ATTRIBUTE_PRINTF_2;
+			    ...)
+  ATTRIBUTE_PRINTF_2 
+  ATTRIBUTE_HIDDEN;
 
-/* Interface to the Bison-generated parser. */
-extern void _gst_yyerror (const char *s);
+/* Interface to the Bison-generated parser.  */
+extern void _gst_yyerror (const char *s)
+  ATTRIBUTE_HIDDEN;
 
 /* Push character 'ic' back into the input queue.  Allows for two
    character pushback currently.  This solves the problem of lexing
    3. and then finding out that what we should have lexed was 3
-   followed by . as a statement terminator. */
-extern void _gst_unread_char (int ic);
+   followed by . as a statement terminator.  */
+extern void _gst_unread_char (int ic) 
+  ATTRIBUTE_HIDDEN;
 
-/* Return the next character from the topmost stream in the stack. */
-extern int _gst_next_char (void);
+/* Return the next character from the topmost stream in the stack.  */
+extern int _gst_next_char (void) 
+  ATTRIBUTE_HIDDEN;
 
-/* Return whether the topmost stream is an interactive one. */
-extern mst_Boolean _gst_get_cur_stream_prompt (void) ATTRIBUTE_PURE;
+/* Return whether the topmost stream is an interactive one.  */
+extern mst_Boolean _gst_get_cur_stream_prompt (void)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
 
-/* Return the type of the topmost stream in the stack. */
-extern stream_type _gst_get_cur_stream_type (void) ATTRIBUTE_PURE;
-
-/* The name of the changelog file. */
-extern char *_gst_change_file_name;
+/* Return the type of the topmost stream in the stack.  */
+extern stream_type _gst_get_cur_stream_type (void)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
 
 /* Return the name of the topmost stream in the stack if it is of type
-   STREAM_FILE; nil otherwise. */
-extern OOP _gst_get_cur_file_name (void) ATTRIBUTE_PURE;
+   STREAM_FILE; nil otherwise.  */
+extern OOP _gst_get_cur_file_name (void)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
 
 /* Return the file position in the topmost stream in the stack if it
-   is of type STREAM_FILE; -1 otherwise. */
-extern int _gst_get_cur_file_pos (void) ATTRIBUTE_PURE;
+   is of type STREAM_FILE; -1 otherwise.  */
+extern off_t _gst_get_cur_file_pos (void)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
 
 /* Returns an OOP containing the string of the topmost stream if it is
-   of type STREAM_STRING; nil otherwise. */
-extern OOP _gst_get_cur_string (void) ATTRIBUTE_PURE;
+   of type STREAM_STRING; nil otherwise.  */
+extern OOP _gst_get_cur_string (void)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
 
 #ifdef HAVE_READLINE
-/* Initialize the completion interface for Readline. */
-extern void _gst_initialize_readline (void);
+/* Initialize the completion interface for Readline.  */
+extern void _gst_initialize_readline (void) 
+  ATTRIBUTE_HIDDEN;
 
 /* Look at the first LEN bytes starting at STR, and add the
    colon-separated keywords, or the whole string if it starts with an
-   uppercase character AND it contains no colons. */
+   uppercase character AND it contains no colons.  */
 extern void _gst_add_symbol_completion (const char *str,
-					int len);
+					int len) 
+  ATTRIBUTE_HIDDEN;
 
 /* Returns a Smalltalk String for the data read by the Readline
    interface, or nil if the stream does not have type
-   STREAM_READLINE. */
-extern OOP _gst_get_cur_readline (void) ATTRIBUTE_PURE;
+   STREAM_READLINE.  */
+extern OOP _gst_get_cur_readline (void)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
 
 /* Add 1 to the completion-enabling flag.  The flag starts at 1
    (completions are active) and completions are enabled until the flag
-   falls below this value. */
-extern void _gst_enable_completion (void);
+   falls below this value.  */
+extern void _gst_enable_completion (void) 
+  ATTRIBUTE_HIDDEN;
 
 /* Subtract 1 from the completion-enabling flag.  The flag starts at 1
    (completions are active) and if completions are disabled, they are
-   not enabled while the flag is below this value. */
-extern void _gst_disable_completion (void);
+   not enabled while the flag is below this value.  */
+extern void _gst_disable_completion (void) 
+  ATTRIBUTE_HIDDEN;
 
 
 #endif /* GST_INPUT_H */
