@@ -35,39 +35,35 @@
 
 /* True if errors must be reported to the standard error, false if
    errors should instead stored so that they are passed to Smalltalk
-   code. */
-extern mst_Boolean _gst_report_errors;
+   code.  */
+extern mst_Boolean _gst_report_errors 
+  ATTRIBUTE_HIDDEN;
 
 /* The location of the first error reported, stored here so that
-   compilation primitives can pass them to Smalltalk code. */
-extern char *_gst_first_error_str, *_gst_first_error_file;
-extern long _gst_first_error_line;
+   compilation primitives can pass them to Smalltalk code.  */
+extern char *_gst_first_error_str 
+  ATTRIBUTE_HIDDEN, *_gst_first_error_file 
+  ATTRIBUTE_HIDDEN;
+extern int _gst_first_error_line 
+  ATTRIBUTE_HIDDEN;
 
-/* This is necessary so that the grammar knows when it must switch to
-   compile mode */
-extern mst_Boolean _gst_compile_code;
+/* The obstack containing parse tree nodes.  */
+extern struct obstack *_gst_compilation_obstack 
+  ATTRIBUTE_HIDDEN;
 
-/* The obstack containing parse tree nodes. */
-extern struct obstack *_gst_compilation_obstack;
+/* Parse the topmost stream in the stack.  */
+extern void _gst_parse_stream (mst_Boolean method)
+  ATTRIBUTE_HIDDEN;
 
-/* Parse the topmost stream in the stack. */
-extern void _gst_parse_stream (void);
-
-/* Lexer interface to the lexer. */
-extern int _gst_yylex (PTR lvalp, YYLTYPE *llocp);
+/* Lexer interface to the lexer.  */
+extern int _gst_yylex (PTR lvalp, YYLTYPE *llocp) 
+  ATTRIBUTE_HIDDEN;
 
 /* Print on file FILE a description of TOKEN, with auxiliary data
-   stored in LVAL. */
+   stored in LVAL.  */
 extern void _gst_yyprint (FILE * file,
 			  int token,
-			  PTR lval);
-
-/* Return the position in the stream where the method had started. */
-extern int _gst_get_method_start_pos (void) ATTRIBUTE_PURE;
-
-/* Clear the starting-position state.  As soon as another token is
-   lexed, the flag is reinitialized to the starting position of the
-   next token. */
-extern void _gst_clear_method_start_pos (void);
+			  PTR lval) 
+  ATTRIBUTE_HIDDEN;
 
 #endif /* GST_LEX_H */
