@@ -42,6 +42,7 @@ typedef enum
   TREE_BINARY_EXPR,
   TREE_KEYWORD_EXPR,
   TREE_VARIABLE_NODE,
+  TREE_ATTRIBUTE_LIST,
   TREE_KEYWORD_LIST,
   TREE_VAR_DECL_LIST,
   TREE_VAR_ASSIGN_LIST,
@@ -71,7 +72,7 @@ typedef struct byte_object
 }
  *byte_object;
 
-/* A forward declaration. */
+/* A forward declaration.  */
 typedef struct tree_node *tree_node;
 
 #include "gst-parse.h"
@@ -85,7 +86,7 @@ typedef struct tree_node *tree_node;
    the nodes is a list of arguments or temporaries).  */
 typedef struct list_node
 {
-  char *name;
+  const char *name;
   tree_node value;
   tree_node next;
   tree_node *nextAddr;
@@ -96,7 +97,7 @@ list_node;
    containing the arguments.  The same data structure is also used for
    assignments (TREE_ASSIGN_EXPR) and in this case RECEIVER is the list
    of assigned-to variables, SELECTOR is dummy and EXPRESSION is the
-   assigned value. */
+   assigned value.  */
 typedef struct expr_node
 {
   tree_node receiver;
@@ -106,7 +107,7 @@ typedef struct expr_node
 expr_node;
 
 /* The different kinds of constants that can be stored in a
-   const_node. */
+   const_node.  */
 typedef enum
 {
   CONST_BYTE_OBJECT,
@@ -121,18 +122,18 @@ typedef enum
 const_type;
 
 /* A parse-tree node holding a constant.  CONSTTYPE identifies which
-   kind of constant is stored, the VAL union can include a long, a
+   kind of constant is stored, the VAL union can include an intptr_t, a
    double, a string (char *), an OOP (typically a Symbol, Association
    or ScaledDecimal), an array (stored as a list_node) or a
-   byte_object struct (for ByteArrays and LargeIntegers). */
+   byte_object struct (for ByteArrays and LargeIntegers).  */
 typedef struct const_node
 {
   const_type constType;
   union
   {
-    long iVal;
+    intptr_t iVal;
     long double fVal;
-    char *sVal;
+    const char *sVal;
     OOP oopVal;
     tree_node aVal;
     byte_object boVal;
@@ -145,18 +146,18 @@ const_node;
    with a nil receiver, holding the selector for the method and
    a list_node (of type TREE_VAR_DECL_LIST) for the arguments.  The
    method's temporaries and statements are also held in list_nodes
-   (respectively, of course, TEMPORARIES and STATEMENTS). */
+   (respectively, of course, TEMPORARIES and STATEMENTS).  */
 typedef struct method_node
 {
   tree_node selectorExpr;
   tree_node temporaries;
-  char *primitiveName;
+  tree_node attributes;
   tree_node statements;
 }
 method_node;
 
 /* A parse-tree node defining a block.  Not having a name, blocks
-   hold arguments in a simple list_node as well. */
+   hold arguments in a simple list_node as well.  */
 typedef struct block_node
 {
   tree_node arguments;
@@ -192,109 +193,129 @@ struct tree_node
 #define v_method	nodeVal.nvMethod
 
 
-/* This is how nil is printed.  */
-extern char *_gst_nil_name;
-
-
 /* Create a method_node with the given fields (see description under
    struct method_node).  TEMPORARIES can possibly be NULL.  If the
-   method has a primitive associated with it, then the PRIMITIVENAME
-   is not NULL.  */
+   method has any attributes associated with it, then they are in
+   ATTRIBUTES.  */
 extern tree_node _gst_make_method (YYLTYPE *location,
 				   tree_node selectorExpr,
 				   tree_node temporaries,
-				   char *primitiveName,
-				   tree_node statements);
+				   tree_node attributes,
+				   tree_node statements)
+  ATTRIBUTE_HIDDEN;
 
 /* Create an expr_node to be passed to _gst_make_method for a unary
    selector, or representing a send of a unary message
-   UNARYSELECTOREXPR to the object identified by RECEIVER. */
+   UNARYSELECTOREXPR to the object identified by RECEIVER.  */
 extern tree_node _gst_make_unary_expr (YYLTYPE *location,
 				       tree_node receiver,
-				       char *unarySelectorExpr);
+				       const char *unarySelectorExpr)
+  ATTRIBUTE_HIDDEN;
 
 /* Create an expr_node to be passed to _gst_make_method for
    a binary selector, or representing a send of a binary message
    BINARYOP to the object identified by RECEIVER, with the given
-   ARGUMENT. */
+   ARGUMENT.  */
 extern tree_node _gst_make_binary_expr (YYLTYPE *location,
 				        tree_node receiver,
-					char *binaryOp,
-					tree_node argument);
+					const char *binaryOp,
+					tree_node argument)
+  ATTRIBUTE_HIDDEN;
 
 /* Create an expr_node to be passed to _gst_make_method for a keyword
    selector, or representing a send of a keyword message identified by
    KEYWORDMESSAGE to the object identified by RECEIVER.  The selector
    is split across the different list_nodes making up KEYWORDMESSAGE
-   and joined at compilation time. */
+   and joined at compilation time.  */
 extern tree_node _gst_make_keyword_expr (YYLTYPE *location,
 				  	 tree_node receiver,
-					 tree_node keywordMessage);
+					 tree_node keywordMessage)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a list_node that represents a variable called NAME (it does
    not matter if it is a global, local, or instance variable, or an
    argument).  */
 extern tree_node _gst_make_variable (YYLTYPE *location,
-				     char *name);
+				     const char *name)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a list_node that represents a part of a keyword selector,
    KEYWORD, together with the corresponding argument EXPRESSION.  */
 extern tree_node _gst_make_keyword_list (YYLTYPE *location,
-				         char *keyword,
-					 tree_node expression);
+				         const char *keyword,
+					 tree_node expression)
+  ATTRIBUTE_HIDDEN;
+
+/* Given a keyword list node, modify it in place to make an attribute list
+   tree node; evaluate all non-constant arguments and return NULL if
+   any evaluation fails.  */
+extern tree_node _gst_make_attribute_list (YYLTYPE *location,
+					   tree_node keywords)
+  ATTRIBUTE_HIDDEN;
 
 /* Given a variable tree node, convert it to a variable list tree node
    with a NULL next link.  */
 extern tree_node _gst_make_variable_list (YYLTYPE *location,
-				  	  tree_node variable);
+				  	  tree_node variable)
+  ATTRIBUTE_HIDDEN;
 
 /* Given a variable tree node, convert it to an assignment list tree node
    with a NULL next link.  */
 extern tree_node _gst_make_assignment_list (YYLTYPE *location,
-				  	    tree_node variable);
+				  	    tree_node variable)
+  ATTRIBUTE_HIDDEN;
 
-/* Create an expr_node of type TREE_ASSIGN_EXPR. */
+/* Create an expr_node of type TREE_ASSIGN_EXPR.  */
 extern tree_node _gst_make_assign (YYLTYPE *location,
 				   tree_node variables,
-				   tree_node expression);
+				   tree_node expression)
+  ATTRIBUTE_HIDDEN;
 
 /* Create an expr_node of type TREE_RETURN_EXPR where the returned
-   EXPRESSION is stored in the RECEIVER field of the node. */
+   EXPRESSION is stored in the RECEIVER field of the node.  */
 extern tree_node _gst_make_return (YYLTYPE *location,
-				   tree_node expression);
+				   tree_node expression)
+  ATTRIBUTE_HIDDEN;
 
-/* Create a const_node storing a long int, IVAL.  */
+/* Create a const_node storing an intptr_t, IVAL.  */
 extern tree_node _gst_make_int_constant (YYLTYPE *location,
-				  	 long int ival);
+				  	 intptr_t ival)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node storing a double, FVAL.  The type (FloatD,
-   FloatE, FloatQ) is given by TYPE. */
+   FloatE, FloatQ) is given by TYPE.  */
 extern tree_node _gst_make_float_constant (YYLTYPE *location,
 				  	   long double fval,
-					   int type);
+					   int type)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node storing a char, CVAL.  */
 extern tree_node _gst_make_char_constant (YYLTYPE *location,
-				  	  char cval);
+				  	  char cval)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node storing a symbol, pointed to by the 
    SYMBOLNODE's NAME member.  The symbol is interned and the
-   const_node is created with its OOP. */
+   const_node is created with its OOP.  */
 extern tree_node _gst_make_symbol_constant (YYLTYPE *location,
-				  	    tree_node symbolNode);
+				  	    tree_node symbolNode)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node storing a string, pointed to by SVAL.  */
 extern tree_node _gst_make_string_constant (YYLTYPE *location,
-				  	    char *sval);
+				  	    const char *sval)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node for an array whose elements are
    described by the nodes in the list, AVAL.  */
 extern tree_node _gst_make_array_constant (YYLTYPE *location,
-				  	   tree_node aval);
+				  	   tree_node aval)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node for the byte_object BOVAL (a LargeInteger).  */
 extern tree_node _gst_make_byte_object_constant (YYLTYPE *location,
-				  	         byte_object boval);
+				  	         byte_object boval)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node for a ByteArray object, creating a
    byteObjectConst out of the single elements which are stored in AVAL
@@ -302,76 +323,89 @@ extern tree_node _gst_make_byte_object_constant (YYLTYPE *location,
    format to byteObjectConst format and answer the resulting
    const_node.  */
 extern tree_node _gst_make_byte_array_constant (YYLTYPE *location,
-				  	        tree_node aval);
+				  	        tree_node aval)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a const_node for an object, OVAL, which is typically a
    ScaledDecimal.  */
 extern tree_node _gst_make_oop_constant (YYLTYPE *location,
-				  	 OOP oval);
+				  	 OOP oval)
+  ATTRIBUTE_HIDDEN;
 
 /* Create an TREE_ARRAY_CONSTRUCTOR node, that is a const_node whose
    aVal does not contain other constants, but rather statements to be
    evaluated at run-time and whose results are to put each in an
    element of the array.  */
 extern tree_node _gst_make_array_constructor (YYLTYPE *location,
-				  	      tree_node statements);
+				  	      tree_node statements)
+  ATTRIBUTE_HIDDEN;
 
 /* Resolve the variable binding to an association and create a
-   const_node of CONST_OOP type. */
+   const_node of CONST_OOP type.  */
 extern tree_node _gst_make_binding_constant (YYLTYPE *location,
-				  	     tree_node variables);
+				  	     tree_node variables)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a TREE_SYMBOL_NODE describing an identifier (variable,
    unary/binary selector or symbol constant, it doesn't patter) pointed
    to by IDENT.  */
 extern tree_node _gst_intern_ident (YYLTYPE *location,
-				    char *ident);
+				    const char *ident)
+  ATTRIBUTE_HIDDEN;
 
 /* Create an element of an array constant, which is a list type object.
    Return the element with the next field NILed out and pointing to
    the first element, ELT.  */
 extern tree_node _gst_make_array_elt (YYLTYPE *location,
-				      tree_node elt);
+				      tree_node elt)
+  ATTRIBUTE_HIDDEN;
 
 /* Creates a block tree node with the given ARGUMENTS, TEMPORARIES and
-   STATEMENTS. */
+   STATEMENTS.  */
 extern tree_node _gst_make_block (YYLTYPE *location,
 				  tree_node arguments,
 				  tree_node temporaries,
-				  tree_node statements);
+				  tree_node statements)
+  ATTRIBUTE_HIDDEN;
 
 /* Creates a node for holding a list of cascaded messages (basically an
    expr_node that isn't using its symbol.  MESSAGEEXPR is the expression
    invoke first as it computes the receiver.  Then the remaining
-   CASCADEDMESSAGES are sent to that same receiver. */
+   CASCADEDMESSAGES are sent to that same receiver.  */
 extern tree_node _gst_make_cascaded_message (YYLTYPE *location,
 				   	     tree_node messageExpr,
-					     tree_node cascadedMessages);
+					     tree_node cascadedMessages)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a node of type TREE_STATEMENT_LIST, where the first node is
    EXPRESSION.  */
 extern tree_node _gst_make_statement_list (YYLTYPE *location,
-					   tree_node expression);
+					   tree_node expression)
+  ATTRIBUTE_HIDDEN;
 
 /* Create a TREE_MESSAGE_LIST which is used as the second parameter to
    _gst_make_cascaded_message -- that is, it represents the sends after
-   the first. */
+   the first.  */
 extern tree_node _gst_make_message_list (YYLTYPE *location,
-				   	 tree_node messageElt);
+				   	 tree_node messageElt)
+  ATTRIBUTE_HIDDEN;
 
 /* Adds node N2 onto a list of nodes headed by N1.  N1 contains the
    address of the last NEXT field in the chain, so storing N2 into
    there indirectly and then making that NEXT field point to N2's NEXT
    field works properly.  */
-extern void _gst_add_node (tree_node n1,
-			   tree_node n2);
+extern tree_node _gst_add_node (tree_node n1,
+				tree_node n2)
+  ATTRIBUTE_HIDDEN;
 
 /* Free the objects on the compilation obstack.  */
-extern void _gst_free_tree ();
+extern void _gst_free_tree ()
+  ATTRIBUTE_HIDDEN;
 
 /* Print the NODE with LEVEL spaces of indentation.  */
 extern void _gst_print_tree (tree_node node,
-			     int level);
+			     int level)
+  ATTRIBUTE_HIDDEN;
 
 
 #endif /* GST_TREE_H */
