@@ -1,5 +1,5 @@
 /* Test that the handler is called, with the right fault address.
-   Copyright (C) 2002-2003  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 2002-2005  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "sigsegv.h"
 #include <stdio.h>
@@ -25,7 +25,7 @@
 
 unsigned long page;
 
-int handler_called = 0;
+volatile int handler_called = 0;
 
 int
 handler (void *fault_address, int serious)
@@ -71,6 +71,14 @@ main ()
       fprintf (stderr, "mprotect failed.\n");
       exit (2);
     }
+  /* Test whether it's possible to make it read-write after it was read-only.
+     This is not possible on Cygwin.  */
+  if (mprotect ((void *) page, 0x4000, PROT_READ_WRITE) < 0
+      || mprotect ((void *) page, 0x4000, PROT_READ) < 0)
+    {
+      fprintf (stderr, "mprotect failed.\n");
+      exit (2);
+    }
 
   /* Install the SIGSEGV handler.  */
   sigsegv_install_handler (&handler);
@@ -84,6 +92,7 @@ main ()
   if (handler_called != 1)
     exit (1);
   /* Test passed!  */
+  printf ("Test passed.\n");
   return 0;
 }
 
